;*******************************************************************************
;
;   Title:  "BLOCK ACID DUB"
;
;   Author: Frantic/Hack'n'Trade
;			alias Glenn Again/Kommando Knorr
;
;   Size:   Code + data + load address = 254 bytes
;           Two bytes reserved for the file system,
;           so the tune will fit in a single disk
;           block.
;
;   Chip:   6581! (PAL)
;
;   Format:	DreamAss was used to assemble this source.
;
;*******************************************************************************
;
; Last Tiny SID compo I aimed to make a long composition with harmonies and a 
; "real" melody, rather than creating advanced SID sounds. This time I aimed for 
; fatter sounds instead. Having explicit or implicit functionality for hardrestart, 
; arpeggio, wavetables and so on. The player is capable of quite flexible creation 
; of sounds. In fact, any combination of SID register values each frame is 
; possible (but only one value per frame, and excluding lobyte of frequency).
;
;*******************************************************************************
; Comments on some of the methods used to decrease size:
;
; - Using "random" recombinations of one single data sequence to make the tune 
;   somewhat varied and "longer" (filter type is also changed over time). A 
;   timer value is used as "random number" and this causes the tune to appear 
;	different each time. This player concept also allows one to use only one 
;   stream of data for all channels (only one "sequence") that just loops all 
;   the time to avoid seq-tables and code for handling such things. The 
;   possibilities of recombination are, however, configurable, so there is a 
;   certain degree of control of what might happen. (Jucke cited som dub guru 
;   the other day. This guru said that one part of the brain follows what is 
;   steady in the music, and another part follows that which is fleeting and 
;   passing, and therebetween a tension arises. According to the guru, that is 
;   what constitutes dub. Hehe.. Anyway, I hope that description fits this 
;   dubish likkle tune of mine. ;)
;
; - I am only using notes which can be approximated by using SID_FRQHI while
;   letting Sid_FreqLo remain set to $00 all the time. That is, using base note
;   of $0400, $0800, $1000, $2000 and so on. See end of source for some more 
;	info on that. There is a html/javascript attached in this package which
;	was used for frequency calculation. Just enter a 16-bit base frequency
;   in the source of the script, and view the results in a www browser.
;
; - AttackDecay is never changed in Voc 1 & 2 and remain set to $00 all the time.
;
; - Using illegal ops whenever suitable (and not just for the sake of doing it
;   of course). SBX, ANC & ASR are used here and they all do "two things in one"
;   apart from SBX, which is useful for subtraction with the X register, rather 
;   than the A-register affected by standard SBC.
;
; - Not using different code structures for "notes" and "instruments". It's
;   all just SID data, and freq-setting can be included in the "instruments"
;   (in the case of a bass drum for example) or just be set on it's own, like
;   a "note".
;
;*******************************************************************************
;           GREETINGS TO NINJA/THE DREAMS, THE 252 BYTE PUNK!
;*******************************************************************************

; CONSTANTS

INSTPTR		= $02 ;= $03 +7 +7 (X offs is 1...)
TICKCOUNT	= $40 ;= $41, $48, $4f (X offs is 1...) ..is set to zero at startup.
SONGPOS		= $99 ;= is set to zero at startup
CTRLBYTE	= $60

NUMBEROFVOICES	= 3 ;All three voices, of course!

;---------------------------------
; CODE

		* = $326
@binbegin:

        .word @start	;Four byte init code to make it an executable.
        .word $f6ed
        
@start:	sei

;--
; Main player loop

@wrast:	cpx $d012 ;X=?? after init and X=$fb after loop.. both values ok..
		bne @wrast

		;Loop once for each SID voice.
		ldx #((NUMBEROFVOICES-1)*7)+1	; +1 because we skip the Sid_FreqLO register altogether. Freeing one control bit.
@vocloop:
		
		lda TICKCOUNT+7+7+1;,x	;..but first, Filter Sweep - controlled by tickcounter from drum channel, to achieve variable behavior.
		asl
		asl
		asl
		sta $d416

		;Handle tick counter
		dec TICKCOUNT,x
		bpl	@parseinst

		;--------------------------------------------
		;FETCH NEXT INSTRUMENT (PARSE NEXT PROBNODE)
@parseseq:
		lda SONGPOS
		inc SONGPOS
		and #%00011111		;There are 32 seqdata bytes so just throw away the three upper bits to take care of wrapping.
		tay					;A reg is set some lines up..

		;Chose one of 4 possible instrument pointers
		lda $dc04			;Get "random" value from $dc04
		.byte $2b, 3		;ANC instruction. Same result as "and #3 / clc" in this case.
		adc @compositiongrid,y
		pha
		and #%00011111		;Clear 3 msb
		tay
		pla
		lsr
		.byte $4b, %01110000 ;Illegal ASR (AND first, to remove instrument ptr bits, and then LSR result, in one instruction.)
		ora #%00000111
		sta TICKCOUNT,x		;Store the new tick counter value
		lda @probnodes,y	;Get instrument number
		beq @nonewinst		;Instrument table pos 0 has the special meaning NO INSTRUMENT CHANGE (and contains no instrument data).
		sta INSTPTR,x		;Store new instrument ptr..
@nonewinst:

		;--------------------------------------------
		;PARSE INSTRUMENT DATA AND WRITE TO SID HERE..		
@parseinst:

		;HARD RESTART - only for drum channel..
		ldy TICKCOUNT+1+7+7
		cpy #2
		bne @nohr			;Skip instrument read and use #2 as instptr instead.
		sty INSTPTR+1+7+7
@nohr:

		;Fetch instrument data
		ldy INSTPTR,x
		lda @instdata,y		;Fetch ctrlbyte.
		sta CTRLBYTE

		stx @sidwlo			;Save voice offset directly in SID-write code.
		txs					;Store X in stackptr temporarily, for later retrieval.
		ldx #5				;6 registers used for each SID Voice.
@parslp:asl CTRLBYTE
		bcc @nxtbyt
		iny
		lda @instdata,y
@sidwlo	= *+1
		sta $d4ff,x
@nxtbyt:dex
		bpl @parslp
	
		tsx					;Restore voice pointer in X..
		
		;Check for STAY parameter. If it's set, then just skip pointer update.
		asl CTRLBYTE
		bmi @noptrupdate
		
		;Save updated instptr (either just increased or from a JUMP argument).
		iny					;This one points either to first step in next instrow or to the jump argument.
		tya
		bcc @nojmpfetch		;Skip Jump-destination fetch if jumpbit was set (carry set by the ASL instruction above).
		lda @instdata,y		;Fetch instjmp argument
@nojmpfetch:
		sta INSTPTR,x		;Now points to next instrument row to be parsed..
@noptrupdate:
@loopend:

		;Loopcounting (X) and filter type changes.
		lda SONGPOS
		.byte $4b,%11100000	;Use ASR to "calculate" filtertype. Don't change filtertype too often, as a straight AND would give.
		bne @dontforce
		lda #$10 			;Force lopass filter if no other filter type is set.
@dontforce:
		ora #$0f			;Set Global Volume (also used as AXS and-value) ;REMOVED.....and always turn bandpass on
		sta $d418			;Filter type / Global volume
		.byte $cb,7			;AXS #7 or SBX #7 depending on who you ask.
		bpl @vocloop
		inx 				;x becomes = $fb
		stx $d417			;Set filter resonance an voice input to #$fb (doesn't matter that we also filter external input by this)
		bmi @wrast
		
;=============================================================================
; Composition Data
;
; Contains a table of pointers to "probability nodes". One row for each voice.

;Duration constants.
	D1 = 0 << 5
	D2 = 1 << 5
	D3 = 2 << 5
	D4 = 3 << 5
	D5 = 4 << 5
	D6 = 5 << 5
	D7 = 6 << 5
	D8 = 7 << 5 ;Max duration.. 

@compositiongrid:
	;		|Voc1			|Voc2			|Voc3		|SeqStep|
	;-------|---------------|---------------|-----------|-------|
	.byte	D2 | @pbd,		D5 | @pabnt,	D2 | @pnote2; 0 #	|first row must set all three channels.
;	.byte												; 1
	.byte	D1 | @psd,						D4 | @pabAAB; 2
	.byte	D1 | @psd									; 3
	.byte	D1 | @pbd									; 4 #
	.byte	D1 | @pbdl,		D4 | @pabAB					; 5
	.byte	D2 | @psd,						D7 | @pabBl	; 6
;	.byte												; 7
	.byte	D2 | @pbd									; 8 #
	.byte					D6 | @synx					; 9
	.byte	D2 | @psd									; a
;	.byte												; b
	.byte	D3 | @pbd									; c #
	.byte									D5 | @pnote	; d
;	.byte												; e
	.byte	D1 | @psd,		D7 | @pabAB					; f
	.byte	D1 | @pbd									;10 #
	.byte	D3 | @psd									;11
	.byte									D6 | @pabAl	;12
;	.byte												;13
	.byte	D3 | @pbd									;14 #
;	.byte												;15
	.byte					D1 | @pabBl					;16
	.byte	D1 | @psd,		D6 | @synn					;17
	.byte	D2 | @pbd,						D6 | @pabBl	;18 #
;	.byte												;19
	.byte	D2 | @psd									;1a
;	.byte												;1b
	.byte	D3 | @pbdll									;1c #
	.byte					D3 | @pabnt					;1d
	.byte									D2 | @pabAAB;1e
	.byte	D1 | @psd									;1f
	;-------|---------------|---------------|-----------|-------|
	
;==========================================================================
@probnodes: ;Probabilitiy "nodes" (4 bytes each) for instrument trig. 
			;Max 32 bytes in this table!

@pbd	= *-@probnodes
		.byte @bd
@pbdl	= *-@probnodes
		.byte @bd
@pbdll	= *-@probnodes
		.byte @bd
@psb	= *-@probnodes
		.byte @bd
@psd	= *-@probnodes
		.byte 0; ;...means no instrument change. 
		.byte @sd
@pabAl	= *-@probnodes
		.byte @n_c2
		.byte @sd
@pabAAB	= *-@probnodes
		.byte @syn
@pabAB	= *-@probnodes
		.byte @abA
@pabBl	= *-@probnodes
		.byte @n_e
		.byte @cpb
@pabnt	= *-@probnodes
		.byte @abAx
		.byte @abB
@synx	= *-@probnodes
		.byte @syn
@pnote	= *-@probnodes
		.byte @abA
		.byte @n_dar
@pnote2	= *-@probnodes
		.byte @n_g
		.byte @n_e
@synn = *-@probnodes
		.byte @n_c2
		.byte @n_gd2
		.byte @syn
		.byte @syn2

;==========================================================================
; "INSTRUMENT EDITOR"
;
; Almost looks like real wavetables, although being a "text file instrument 
; editor", doesn't it? ;)
;
; Some control contants.

	TICK		= %0		;An empty control byte simply causes a delay of one frame.
	SID_SR		= %10000000
	SID_AD		= %01000000
	SID_CTRL	= %00100000
	SID_PULHI	= %00010000
	SID_PULLO	= %00001000
	SID_FRQHI	= %00000100 ;Note that FreqLO is not used at all
	STAY		= %00000001 ;STAY overrides JUMP, so only use one at a time. Else data will get out of sync.
	JUMP		= %00000010

@instdata = *-2:  	;Read comment for hardrestart below to get the idea with the "-2" here.
					;Also, position 0 in the table is used as a "NO INSTRMENT CHANGE FLAG"
					;and shall not contain data anyway.

;--- HARDRESTART
@hardrestart: ;This one should be placed the same ammounts of bytes into the instdata table as ticks to preceed instrument trig.
	.byte	SID_SR |				SID_CTRL |								STAY
	.byte	$00,					$10

@bd	= *-@instdata
	.byte	SID_SR | SID_AD |		SID_CTRL
	.byte	$fb,	$04,			$49
	.byte							SID_CTRL |	SID_PULHI | SID_FRQHI
	.byte							$41,		$08,		$08
	.byte							SID_CTRL
	.byte							$11
	.byte							SID_CTRL |				SID_FRQHI
	.byte							$10,					$07
	.byte													SID_FRQHI | 	STAY
	.byte													$04

;----
@sd = *-@instdata
	.byte	SID_SR |				SID_CTRL
	.byte	$33,					$49
	.byte							SID_CTRL |				SID_FRQHI
	.byte							$41,					$0c
	.byte							SID_CTRL |				SID_FRQHI
	.byte							$80,					$a0
	.byte							SID_CTRL |				SID_FRQHI | 	STAY
	.byte							$40,					$18

;---- Mongo Bass
@cpb = *-@instdata
	.byte	SID_SR |										SID_FRQHI
	.byte	$54,											$06
	.byte							SID_CTRL |				SID_FRQHI |		JUMP
	.byte							$11,					$05,			@cpb

;--- NOTE E (round error 10)
@n_e = *-@instdata
	.byte													SID_FRQHI | 	STAY
	.byte													$05

;--- NOTE G (round error -2)
@n_g = *-@instdata
	.byte													SID_FRQHI | 	STAY
	.byte													$06

;--- ARPEGGIO G - D2 (D2 round error -6, G error -4..)
@n_gd2 = *-@instdata
	.byte													SID_FRQHI
	.byte													$0c
@n_dar = *-@instdata
	.byte													SID_FRQHI |		JUMP
	.byte													$09,			@n_gd2

;--- NOTE C2 (round error 0)
@n_c2 = *-@instdata
	.byte													SID_FRQHI | 	STAY
	.byte													$08

;--- ACID BASS #1
@abA = *-@instdata
	.byte													SID_FRQHI 
	.byte													$04
@abAx = *-@instdata
	.byte	SID_SR |				SID_CTRL |							 	JUMP
	.byte	$9a,					$21,									@abB_entry2

;--- ACID BASS #2
@abB = *-@instdata
	.byte	SID_SR |				SID_CTRL
	.byte	$b8,					$81 ;Just to add some subtle filtered "percussion" here and there.. ;)
@abB_entry2 = *-@instdata
	.byte																	TICK
	.byte							SID_CTRL |								STAY
	.byte							$20

;--- Tri Synth
@syn2 = *-@instdata 
	.byte													SID_FRQHI 
	.byte													$18 ;High G note..
@syn = *-@instdata 
	.byte	SID_SR |				SID_CTRL
	.byte	$fc,					$11
	.byte							SID_CTRL |								STAY
	.byte							$10

;==========================================================================
;PRINT SIZE (Adding two for the obligatory load adress + two for the file system).

;#print (*-@binbegin)+2+2

;==========================================================================
; Here is a list of calculated notefrequencies, assuming that base notes
; of the scale start at $0400 (C), $0800 (C2), $1000 (C3..), $2000, $4000...
; Some of them can be approximated by setting the hibyte of freq to
; the value, and just having lobyte freq set to $00 (as it is when the
; C64 is reset.) These values are calculated with the html/javascript
; file which is attached in the package.

;.byte 4 ; C error: 0  <------------
;.byte 4 ; C# error: 60
;.byte 4 ; D error: 125
;.byte 4 ; D# error: 193
;.byte 5 ; E error: 10  <------------
;.byte 5 ; F error: 86
;.byte 5 ; F# error: 168
;.byte 6 ; G error: -2 <--------------
;.byte 6 ; G# error: 89
;.byte 6 ; A error: 186
;.byte 7 ; A# error: 32
;.byte 7 ; H error: 141
;.byte 8 ; C error: 0 <------------
;.byte 8 ; C# error: 121
;.byte 9 ; D error: -6  <-------------
;.byte 9 ; D# error: 131
;.byte 10 ; E error: 20
;.byte 10 ; F error: 173
;.byte 11 ; F# error: 80
;.byte 12 ; G error: -4  <------------
;.byte 12 ; G# error: 178
;.byte 13 ; A error: 116
;.byte 14 ; A# error: 65
;.byte 15 ; H error: 26
;.byte 16 ; C error: 0  <------------
; byte 16 ; C# error: 243
;.byte 17 ; D error: 245
;.byte 19 ; D# error: 6 <-- (Useable, but not used here..)
;.byte 20 ; E error: 40
;.byte 21 ; F error: 91
;.byte 22 ; F# error: 160
;.byte 24 ; G error: -7   <------------
;.byte 25 ; G# error: 101
;.byte 26 ; A error: 232
;.byte 28 ; A# error: 130
;.byte 30 ; H error: 52
;.byte 32 ; C2 error: 0
